@@ -4,9 +4,10 @@ use crate::compile::{CompileTarget, LIB_CRATE_TYPES};
 use crate::cross_compile::{find_sysconfigdata, parse_sysconfigdata};
 use crate::project_layout::ProjectResolver;
 use crate::pyproject_toml::ToolMaturin;
-use crate::python_interpreter::{InterpreterConfig, InterpreterKind, MINIMUM_PYTHON_MINOR};
-use crate::{BuildContext, PythonInterpreter, Target};
+use crate::python_interpreter::{InterpreterConfig, InterpreterKind};
+use crate::{Bindings, BuildContext, PythonInterpreter, Target};
 use anyhow::{bail, format_err, Context, Result};
+use cargo_metadata::{CrateType, TargetKind};
 use cargo_metadata::{Metadata, Node};
 use cargo_options::heading;
 use pep440_rs::VersionSpecifiers;
@@ -23,22 +24,6 @@ use tracing::{debug, instrument};
 // and more restrictive.
 const PYO3_BINDING_CRATES: [&str; 2] = ["pyo3-ffi", "pyo3"];
 
-fn pyo3_minimum_python_minor_version(major_version: u64, minor_version: u64) -> Option<usize> {
-    if (major_version, minor_version) >= (0, 16) {
-        Some(7)
-    } else {
-        None
-    }
-}
-
-fn pyo3_ffi_minimum_python_minor_version(major_version: u64, minor_version: u64) -> Option<usize> {
-    if (major_version, minor_version) >= (0, 16) {
-        pyo3_minimum_python_minor_version(major_version, minor_version)
-    } else {
-        None
-    }
-}
-
 /// Cargo options for the build process
 #[derive(Debug, Default, Serialize, Deserialize, clap::Parser, Clone, Eq, PartialEq)]
 #[serde(default, rename_all = "kebab-case")]
@@ -232,7 +217,12 @@ impl BuildOptions {
         generate_import_lib: bool,
     ) -> Result<Vec<PythonInterpreter>> {
         match bridge {
-            BridgeModel::Bindings(binding_name, _) | BridgeModel::Bin(Some((binding_name, _))) => {
+            BridgeModel::Bindings(Bindings {
+                name: binding_name, ..
+            })
+            | BridgeModel::Bin(Some(Bindings {
+                name: binding_name, ..
+            })) => {
                 let mut interpreters = Vec::new();
                 if let Some(config_file) = env::var_os("PYO3_CONFIG_FILE") {
                     if !binding_name.starts_with("pyo3") {
@@ -324,8 +314,12 @@ impl BuildOptions {
                                 bail!("{} is not a valid python interpreter", interp.display());
                             }
                         }
-                        interpreters =
-                            find_interpreter_in_sysconfig(interpreter, target, requires_python)?;
+                        interpreters = find_interpreter_in_sysconfig(
+                            bridge,
+                            interpreter,
+                            target,
+                            requires_python,
+                        )?;
                         if interpreters.is_empty() {
                             bail!(
                                 "Couldn't find any python interpreters from '{}'. Please check that both major and minor python version have been specified in -i/--interpreter.",
@@ -362,13 +356,28 @@ impl BuildOptions {
             }
             BridgeModel::Bin(None) | BridgeModel::UniFfi => Ok(vec![]),
             BridgeModel::BindingsAbi3(major, minor) => {
+                let found_interpreters =
+                    find_interpreter_in_host(bridge, interpreter, target, requires_python)
+                        .or_else(|err| {
+                            // Can only use sysconfig-derived interpreter on windows if generating the import lib
+                            if target.is_windows() && !generate_import_lib {
+                                return Err(err.context("Need a Python interpreter to compile for Windows without PyO3's `generate-import-lib` feature"));
+                            }
+
+                            let interps =
+                                find_interpreter_in_sysconfig(bridge,interpreter, target, requires_python)
+                                    .unwrap_or_default();
+                            if interps.is_empty() && !self.interpreter.is_empty() {
+                                // Print error when user supplied `--interpreter` option
+                                Err(err)
+                            } else {
+                                Ok(interps)
+                            }
+                        })?;
+
                 if target.is_windows() {
-                    // Ideally, we wouldn't want to use any python interpreter without abi3 at all.
-                    // Unfortunately, on windows we need one to figure out base_prefix for a linker
-                    // argument.
-                    let interpreters =
-                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
-                            .unwrap_or_default();
+                    // On windows we might need a Python executable to locate a base prefix for
+                    // linker args, if we're not using PyO3's `generate-import-lib` feature.
                     if env::var_os("PYO3_CROSS_LIB_DIR").is_some() {
                         // PYO3_CROSS_LIB_DIR should point to the `libs` directory inside base_prefix
                         // when cross compiling, so we fake a python interpreter matching it
@@ -394,48 +403,36 @@ impl BuildOptions {
                             InterpreterConfig::from_pyo3_config(config_file.as_ref(), target)
                                 .context("Invalid PYO3_CONFIG_FILE")?;
                         Ok(vec![PythonInterpreter::from_config(interpreter_config)])
-                    } else if let Some(interp) = interpreters.first() {
-                        eprintln!("🐍 Using {interp} to generate to link bindings (With abi3, an interpreter is only required on windows)");
-                        Ok(interpreters)
                     } else if generate_import_lib {
-                        eprintln!("🐍 Not using a specific python interpreter (Automatically generating windows import library)");
-                        // fake a python interpreter
-                        Ok(vec![PythonInterpreter {
-                            config: InterpreterConfig {
-                                major: *major as usize,
-                                minor: *minor as usize,
-                                interpreter_kind: InterpreterKind::CPython,
-                                abiflags: "".to_string(),
-                                ext_suffix: ".pyd".to_string(),
-                                pointer_width: None,
-                                gil_disabled: false,
-                            },
-                            executable: PathBuf::new(),
-                            platform: None,
-                            runnable: false,
-                            implementation_name: "cpython".to_string(),
-                            soabi: None,
-                        }])
+                        eprintln!("🐍 Not using a specific python interpreter (automatically generating windows import library)");
+                        let mut found_interpreters = found_interpreters;
+                        // fake a python interpreter if none directly found
+                        if found_interpreters.is_empty() {
+                            found_interpreters.push(PythonInterpreter {
+                                config: InterpreterConfig {
+                                    major: *major as usize,
+                                    minor: *minor as usize,
+                                    interpreter_kind: InterpreterKind::CPython,
+                                    abiflags: "".to_string(),
+                                    ext_suffix: ".pyd".to_string(),
+                                    pointer_width: None,
+                                    gil_disabled: false,
+                                },
+                                executable: PathBuf::new(),
+                                platform: None,
+                                runnable: false,
+                                implementation_name: "cpython".to_string(),
+                                soabi: None,
+                            })
+                        }
+                        Ok(found_interpreters)
                     } else {
-                        bail!("Failed to find a python interpreter");
+                        if found_interpreters.is_empty() {
+                            bail!("Failed to find any python interpreter");
+                        }
+                        Ok(found_interpreters)
                     }
                 } else {
-                    let found_interpreters =
-                        find_interpreter_in_host(bridge, interpreter, target, requires_python)
-                            .or_else(|err| {
-                                let interps = find_interpreter_in_sysconfig(
-                                    interpreter,
-                                    target,
-                                    requires_python,
-                                )
-                                .unwrap_or_default();
-                                if interps.is_empty() && !self.interpreter.is_empty() {
-                                    // Print error when user supplied `--interpreter` option
-                                    Err(err)
-                                } else {
-                                    Ok(interps)
-                                }
-                            })?;
                     eprintln!("🐍 Not using a specific python interpreter");
                     if self.interpreter.is_empty() {
                         // Fake one to make `BuildContext::build_wheels` happy for abi3 when no cpython/pypy found on host
@@ -473,6 +470,7 @@ impl BuildOptions {
                         // we cannot use host pypy so switch to bundled sysconfig instead
                         if !pypys.is_empty() {
                             interps.extend(find_interpreter_in_sysconfig(
+                                bridge,
                                 &pypys,
                                 target,
                                 requires_python,
@@ -881,7 +879,7 @@ fn filter_cargo_targets(
                             .all(|f| resolved_features.contains(f))
                 }
             }
-            _ => target.kind.contains(&"cdylib".to_string()),
+            _ => target.crate_types.contains(&CrateType::CDyLib),
         })
         .map(|target| CompileTarget {
             target: target.clone(),
@@ -893,9 +891,9 @@ fn filter_cargo_targets(
         // Let's try compile one of the target with `--crate-type cdylib`
         let lib_target = root_pkg.targets.iter().find(|target| {
             target
-                .kind
+                .crate_types
                 .iter()
-                .any(|k| LIB_CRATE_TYPES.contains(&k.as_str()))
+                .any(|crate_type| LIB_CRATE_TYPES.contains(crate_type))
         });
         if let Some(target) = lib_target {
             targets.push(CompileTarget {
@@ -911,7 +909,7 @@ fn filter_cargo_targets(
             config_targets.iter().any(|config_target| {
                 let name_eq = config_target.name == target.name;
                 match &config_target.kind {
-                    Some(kind) => name_eq && target.kind.contains(kind),
+                    Some(kind) => name_eq && target.crate_types.contains(&CrateType::from(*kind)),
                     None => name_eq,
                 }
             })
@@ -1013,19 +1011,25 @@ fn is_generating_import_lib(cargo_metadata: &Metadata) -> Result<bool> {
 fn find_bindings(
     deps: &HashMap<&str, &Node>,
     packages: &HashMap<&str, &cargo_metadata::Package>,
-) -> Option<(String, usize)> {
+) -> Option<Bindings> {
     if deps.get("pyo3").is_some() {
-        let ver = &packages["pyo3"].version;
-        let minor =
-            pyo3_minimum_python_minor_version(ver.major, ver.minor).unwrap_or(MINIMUM_PYTHON_MINOR);
-        Some(("pyo3".to_string(), minor))
+        let version = packages["pyo3"].version.clone();
+        Some(Bindings {
+            name: "pyo3".to_string(),
+            version,
+        })
     } else if deps.get("pyo3-ffi").is_some() {
-        let ver = &packages["pyo3-ffi"].version;
-        let minor = pyo3_ffi_minimum_python_minor_version(ver.major, ver.minor)
-            .unwrap_or(MINIMUM_PYTHON_MINOR);
-        Some(("pyo3-ffi".to_string(), minor))
+        let version = packages["pyo3-ffi"].version.clone();
+        Some(Bindings {
+            name: "pyo3-ffi".to_string(),
+            version,
+        })
     } else if deps.contains_key("uniffi") {
-        Some(("uniffi".to_string(), MINIMUM_PYTHON_MINOR))
+        let version = packages["uniffi"].version.clone();
+        Some(Bindings {
+            name: "uniffi".to_string(),
+            version,
+        })
     } else {
         None
     }
@@ -1063,11 +1067,17 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
         .iter()
         .filter(|target| {
             target.kind.iter().any(|kind| {
-                kind != "example" && kind != "test" && kind != "bench" && kind != "custom-build"
+                !matches!(
+                    kind,
+                    TargetKind::Bench
+                        | TargetKind::CustomBuild
+                        | TargetKind::Example
+                        | TargetKind::ProcMacro
+                        | TargetKind::Test
+                )
             })
         })
-        .flat_map(|target| target.crate_types.iter())
-        .map(String::as_str)
+        .flat_map(|target| target.crate_types.iter().cloned())
         .collect();
 
     let bridge = if let Some(bindings) = bridge {
@@ -1078,7 +1088,7 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
         } else if bindings == "bin" {
             // uniffi bindings don't support bin
             let bindings =
-                find_bindings(&deps, &packages).filter(|(bindings, _)| bindings != "uniffi");
+                find_bindings(&deps, &packages).filter(|bindings| bindings.name != "uniffi");
             BridgeModel::Bin(bindings)
         } else {
             if !deps.contains_key(bindings) {
@@ -1088,24 +1098,28 @@ pub fn find_bridge(cargo_metadata: &Metadata, bridge: Option<&str>) -> Result<Br
                 );
             }
 
-            BridgeModel::Bindings(bindings.to_string(), MINIMUM_PYTHON_MINOR)
+            let version = packages[bindings].version.clone();
+            BridgeModel::Bindings(Bindings {
+                name: bindings.to_string(),
+                version,
+            })
         }
-    } else if let Some((bindings, minor)) = find_bindings(&deps, &packages) {
-        if !targets.contains(&"cdylib") && targets.contains(&"bin") {
-            if bindings == "uniffi" {
+    } else if let Some(bindings) = find_bindings(&deps, &packages) {
+        if !targets.contains(&CrateType::CDyLib) && targets.contains(&CrateType::Bin) {
+            if bindings.name == "uniffi" {
                 // uniffi bindings don't support bin
                 BridgeModel::Bin(None)
             } else {
-                BridgeModel::Bin(Some((bindings, minor)))
+                BridgeModel::Bin(Some(bindings))
             }
-        } else if bindings == "uniffi" {
+        } else if bindings.name == "uniffi" {
             BridgeModel::UniFfi
         } else {
-            BridgeModel::Bindings(bindings, minor)
+            BridgeModel::Bindings(bindings)
         }
-    } else if targets.contains(&"cdylib") {
+    } else if targets.contains(&CrateType::CDyLib) {
         BridgeModel::Cffi
-    } else if targets.contains(&"bin") {
+    } else if targets.contains(&CrateType::Bin) {
         BridgeModel::Bin(find_bindings(&deps, &packages))
     } else {
         bail!("Couldn't detect the binding type; Please specify them with --bindings/-b")
@@ -1186,7 +1200,7 @@ fn find_interpreter(
         }
         if !missing.is_empty() {
             let sysconfig_interps =
-                find_interpreter_in_sysconfig(&missing, target, requires_python)?;
+                find_interpreter_in_sysconfig(bridge, &missing, target, requires_python)?;
             found_interpreters.extend(sysconfig_interps);
         }
     } else {
@@ -1242,12 +1256,17 @@ fn find_interpreter_in_host(
 
 /// Find python interpreters in the bundled sysconfig
 fn find_interpreter_in_sysconfig(
+    bridge: &BridgeModel,
     interpreter: &[PathBuf],
     target: &Target,
     requires_python: Option<&VersionSpecifiers>,
 ) -> Result<Vec<PythonInterpreter>> {
     if interpreter.is_empty() {
-        return Ok(PythonInterpreter::find_by_target(target, requires_python));
+        return Ok(PythonInterpreter::find_by_target(
+            target,
+            requires_python,
+            Some(bridge),
+        ));
     }
     let mut interpreters = Vec::new();
     for interp in interpreter {
@@ -1479,11 +1498,11 @@ mod test {
 
         assert!(matches!(
             find_bridge(&pyo3_mixed, None),
-            Ok(BridgeModel::Bindings(..))
+            Ok(BridgeModel::Bindings { .. })
         ));
         assert!(matches!(
             find_bridge(&pyo3_mixed, Some("pyo3")),
-            Ok(BridgeModel::Bindings(..))
+            Ok(BridgeModel::Bindings { .. })
         ));
     }
 
@@ -1521,7 +1540,7 @@ mod test {
 
         assert!(matches!(
             find_bridge(&pyo3_pure, None).unwrap(),
-            BridgeModel::Bindings(..)
+            BridgeModel::Bindings { .. }
         ));
     }
 
@@ -1565,11 +1584,11 @@ mod test {
             .unwrap();
         assert!(matches!(
             find_bridge(&pyo3_bin, Some("bin")).unwrap(),
-            BridgeModel::Bin(Some((..)))
+            BridgeModel::Bin(Some(_))
         ));
         assert!(matches!(
             find_bridge(&pyo3_bin, None).unwrap(),
-            BridgeModel::Bin(Some(..))
+            BridgeModel::Bin(Some(_))
         ));
     }
 
