@@ -175,7 +175,7 @@ impl GenerateCI {
             || matches!(
                 bridge_model,
                 BridgeModel::Bin(Some(_))
-                    | BridgeModel::Bindings(..)
+                    | BridgeModel::Bindings { .. }
                     | BridgeModel::BindingsAbi3(..)
                     | BridgeModel::Cffi
                     | BridgeModel::UniFfi
@@ -389,12 +389,12 @@ jobs:\n",
                 // build free-threaded wheel for python3.13t
                 conf.push_str(&format!(
                     "      - name: Build free-threaded wheels
-          uses: PyO3/maturin-action@v1
-          with:
-            target: ${{{{ matrix.platform.target }}}}
-            args: --release --out dist{maturin_args} -i python3.13t
-            sccache: 'true'
-  "
+        uses: PyO3/maturin-action@v1
+        with:
+          target: ${{{{ matrix.platform.target }}}}
+          args: --release --out dist{maturin_args} -i python3.13t
+          sccache: 'true'
+"
                 ));
                 if !maturin_action_args.is_empty() {
                     conf.push_str(&format!("          {maturin_action_args}\n"));
@@ -629,7 +629,7 @@ jobs:\n",
         }
         conf.push_str(
             r#"      - name: Publish to PyPI
-        if: "startsWith(github.ref, 'refs/tags/')"
+        if: ${{ startsWith(github.ref, 'refs/tags/') }}
         uses: PyO3/maturin-action@v1
         env:
           MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
@@ -665,15 +665,19 @@ jobs:\n",
 #[cfg(test)]
 mod tests {
     use super::GenerateCI;
-    use crate::BridgeModel;
+    use crate::{Bindings, BridgeModel};
     use expect_test::expect;
+    use semver::Version;
 
     #[test]
     fn test_generate_github() {
         let conf = GenerateCI::default()
             .generate_github(
                 "example",
-                &BridgeModel::Bindings("pyo3".to_string(), 7),
+                &BridgeModel::Bindings(Bindings {
+                    name: "pyo3".to_string(),
+                    version: Version::new(0, 23, 0),
+                }),
                 true,
             )
             .unwrap()
@@ -851,7 +855,7 @@ mod tests {
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
-                    if: "startsWith(github.ref, 'refs/tags/')"
+                    if: ${{ startsWith(github.ref, 'refs/tags/') }}
                     uses: PyO3/maturin-action@v1
                     env:
                       MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
@@ -917,12 +921,12 @@ mod tests {
                       sccache: 'true'
                       manylinux: auto
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                        manylinux: auto
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                      manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -955,12 +959,12 @@ mod tests {
                       sccache: 'true'
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                        manylinux: musllinux_1_2
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                      manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -989,12 +993,12 @@ mod tests {
                       args: --release --out dist
                       sccache: 'true'
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                    - name: Upload wheels
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                  - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
                       name: wheels-windows-${{ matrix.platform.target }}
@@ -1021,12 +1025,12 @@ mod tests {
                       args: --release --out dist
                       sccache: 'true'
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                    - name: Upload wheels
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                  - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
                       name: wheels-macos-${{ matrix.platform.target }}
@@ -1051,7 +1055,7 @@ mod tests {
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
-                    if: "startsWith(github.ref, 'refs/tags/')"
+                    if: ${{ startsWith(github.ref, 'refs/tags/') }}
                     uses: PyO3/maturin-action@v1
                     env:
                       MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
@@ -1120,12 +1124,12 @@ mod tests {
                       sccache: 'true'
                       manylinux: auto
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                        manylinux: auto
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                      manylinux: auto
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1158,12 +1162,12 @@ mod tests {
                       sccache: 'true'
                       manylinux: musllinux_1_2
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                        manylinux: musllinux_1_2
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                      manylinux: musllinux_1_2
                   - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
@@ -1192,12 +1196,12 @@ mod tests {
                       args: --release --out dist
                       sccache: 'true'
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                    - name: Upload wheels
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                  - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
                       name: wheels-windows-${{ matrix.platform.target }}
@@ -1224,12 +1228,12 @@ mod tests {
                       args: --release --out dist
                       sccache: 'true'
                   - name: Build free-threaded wheels
-                      uses: PyO3/maturin-action@v1
-                      with:
-                        target: ${{ matrix.platform.target }}
-                        args: --release --out dist -i python3.13t
-                        sccache: 'true'
-                    - name: Upload wheels
+                    uses: PyO3/maturin-action@v1
+                    with:
+                      target: ${{ matrix.platform.target }}
+                      args: --release --out dist -i python3.13t
+                      sccache: 'true'
+                  - name: Upload wheels
                     uses: actions/upload-artifact@v4
                     with:
                       name: wheels-macos-${{ matrix.platform.target }}
@@ -1248,7 +1252,7 @@ mod tests {
                 steps:
                   - uses: actions/download-artifact@v4
                   - name: Publish to PyPI
-                    if: "startsWith(github.ref, 'refs/tags/')"
+                    if: ${{ startsWith(github.ref, 'refs/tags/') }}
                     uses: PyO3/maturin-action@v1
                     env:
                       MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
@@ -1268,7 +1272,10 @@ mod tests {
         let conf = gen
             .generate_github(
                 "example",
-                &BridgeModel::Bindings("pyo3".to_string(), 7),
+                &BridgeModel::Bindings(Bindings {
+                    name: "pyo3".to_string(),
+                    version: Version::new(0, 23, 0),
+                }),
                 true,
             )
             .unwrap()
@@ -1519,7 +1526,7 @@ mod tests {
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
-                    if: "startsWith(github.ref, 'refs/tags/')"
+                    if: ${{ startsWith(github.ref, 'refs/tags/') }}
                     uses: PyO3/maturin-action@v1
                     env:
                       MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
@@ -1695,7 +1702,7 @@ mod tests {
                     with:
                       subject-path: 'wheels-*/*'
                   - name: Publish to PyPI
-                    if: "startsWith(github.ref, 'refs/tags/')"
+                    if: ${{ startsWith(github.ref, 'refs/tags/') }}
                     uses: PyO3/maturin-action@v1
                     env:
                       MATURIN_PYPI_TOKEN: ${{ secrets.PYPI_API_TOKEN }}
