@@ -2,16 +2,20 @@ use crate::module_writer::{add_data, ModuleWriter};
 use crate::pyproject_toml::SdistGenerator;
 use crate::{pyproject_toml::Format, BuildContext, PyProjectToml, SDistWriter};
 use anyhow::{bail, Context, Result};
+use cargo_metadata::camino::Utf8Path;
 use cargo_metadata::{Metadata, MetadataCommand, PackageId};
 use fs_err as fs;
 use ignore::overrides::Override;
 use normpath::PathExt as _;
 use path_slash::PathExt as _;
 use std::collections::HashMap;
+use std::ffi::OsStr;
+use std::io::Write;
 use std::path::{Path, PathBuf};
 use std::process::Command;
 use std::str;
-use tracing::debug;
+use toml_edit::DocumentMut;
+use tracing::{debug, trace};
 
 /// Path dependency information.
 /// It may be in a different workspace than the root crate.
@@ -45,13 +49,14 @@ fn parse_toml_file(path: &Path, kind: &str) -> Result<toml_edit::DocumentMut> {
 /// We only want to add path dependencies that are actually used
 /// to reduce the size of the source distribution.
 fn rewrite_cargo_toml(
-    manifest_path: impl AsRef<Path>,
+    document: &mut DocumentMut,
+    manifest_path: &Path,
     known_path_deps: &HashMap<String, PathDependency>,
-) -> Result<String> {
-    let manifest_path = manifest_path.as_ref();
-    debug!("Rewriting Cargo.toml at {}", manifest_path.display());
-    let mut document = parse_toml_file(manifest_path, "Cargo.toml")?;
-
+) -> Result<()> {
+    debug!(
+        "Rewriting Cargo.toml `workspace.members` at {}",
+        manifest_path.display()
+    );
     // Update workspace members
     if let Some(workspace) = document.get_mut("workspace").and_then(|x| x.as_table_mut()) {
         if let Some(members) = workspace.get_mut("members").and_then(|x| x.as_array()) {
@@ -100,7 +105,34 @@ fn rewrite_cargo_toml(
             }
         }
     }
-    Ok(document.to_string())
+    Ok(())
+}
+
+/// Rewrite `Cargo.toml` to find the readme in the same directory.
+///
+/// `package.readme` may point to any point above the package, so when we move the directory, but
+/// keep the readme position, we could get different readme files at the same archive location.
+/// Putting the readme in the same directory as the `Cargo.toml` prevents this.
+fn rewrite_cargo_toml_readme(
+    document: &mut DocumentMut,
+    manifest_path: &Path,
+    readme_name: Option<&str>,
+) -> Result<()> {
+    debug!(
+        "Rewriting Cargo.toml `package.readme` at {}",
+        manifest_path.display()
+    );
+
+    if let Some(readme_name) = readme_name {
+        let project = document.get_mut("package").with_context(|| {
+            format!(
+                "Missing `[package]` table in Cargo.toml with readme at {}",
+                manifest_path.display()
+            )
+        })?;
+        project["readme"] = toml_edit::value(readme_name);
+    }
+    Ok(())
 }
 
 /// When `pyproject.toml` is inside the Cargo workspace root,
@@ -148,13 +180,20 @@ fn add_crate_to_source_distribution(
     writer: &mut SDistWriter,
     manifest_path: impl AsRef<Path>,
     prefix: impl AsRef<Path>,
+    readme: Option<&Path>,
     known_path_deps: &HashMap<String, PathDependency>,
     root_crate: bool,
     skip_cargo_toml: bool,
 ) -> Result<()> {
+    debug!(
+        "Getting cargo package file list for {}",
+        manifest_path.as_ref().display()
+    );
+    let prefix = prefix.as_ref();
     let manifest_path = manifest_path.as_ref();
+    let args = ["package", "--list", "--allow-dirty", "--manifest-path"];
     let output = Command::new("cargo")
-        .args(["package", "--list", "--allow-dirty", "--manifest-path"])
+        .args(args)
         .arg(manifest_path)
         .output()
         .with_context(|| {
@@ -172,38 +211,54 @@ fn add_crate_to_source_distribution(
             String::from_utf8_lossy(&output.stderr),
         );
     }
+    if !output.stderr.is_empty() {
+        eprintln!(
+            "From `cargo {} {}`:",
+            args.join(" "),
+            manifest_path.display()
+        );
+        std::io::stderr().write_all(&output.stderr)?;
+    }
 
-    let file_list: Vec<&Path> = str::from_utf8(&output.stdout)
+    let file_list: Vec<&str> = str::from_utf8(&output.stdout)
         .context("Cargo printed invalid utf-8 ಠ_ಠ")?
         .lines()
-        .map(Path::new)
         .collect();
 
+    trace!("File list: {}", file_list.join(", "));
+
     // manifest_dir should be a relative path
     let manifest_dir = manifest_path.parent().unwrap();
-    let target_source: Vec<(PathBuf, PathBuf)> = file_list
-        .iter()
+    let target_source: Vec<_> = file_list
+        .into_iter()
         .map(|relative_to_manifests| {
             let relative_to_cwd = manifest_dir.join(relative_to_manifests);
-            (relative_to_manifests.to_path_buf(), relative_to_cwd)
+            (relative_to_manifests, relative_to_cwd)
         })
-        // We rewrite Cargo.toml and add it separately
         .filter(|(target, source)| {
             #[allow(clippy::if_same_then_else)]
-            // Skip generated files. See https://github.com/rust-lang/cargo/issues/7938#issuecomment-593280660
-            // and https://github.com/PyO3/maturin/issues/449
-            if target == Path::new("Cargo.toml.orig") || target == Path::new("Cargo.toml") {
+            if *target == "Cargo.toml.orig" {
+                // Skip generated files. See https://github.com/rust-lang/cargo/issues/7938#issuecomment-593280660
+                // and https://github.com/PyO3/maturin/issues/449
                 false
-            } else if root_crate && target == Path::new("pyproject.toml") {
-                // pyproject.toml is handled separately because it has be to put in the root dir
+            } else if *target == "Cargo.toml" {
+                // We rewrite Cargo.toml and add it separately
+                false
+            } else if root_crate && *target == "pyproject.toml" {
+                // pyproject.toml is handled separately because it has to be put in the root dir
                 // of source distribution
                 false
-            } else if matches!(target.extension(), Some(ext) if ext == "pyc" || ext == "pyd" || ext == "so") {
+            } else if prefix.components().count() == 1 && *target == "pyproject.toml" {
+                // Skip pyproject.toml for cases when the root is in a workspace member and both the
+                // member and the root have a pyproject.toml.
+                debug!("Skipping potentially non-main {}", prefix.join(target).display());
+                false
+            } else if matches!(Path::new(target).extension(), Some(ext) if ext == "pyc" || ext == "pyd" || ext == "so") {
                 // Technically, `cargo package --list` should handle this,
                 // but somehow it doesn't on Alpine Linux running in GitHub Actions,
                 // so we do it manually here.
                 // See https://github.com/PyO3/maturin/pull/1255#issuecomment-1308838786
-                debug!("Ignoring {}", target.display());
+                debug!("Ignoring {}", target);
                 false
             } else {
                 source.exists()
@@ -211,20 +266,37 @@ fn add_crate_to_source_distribution(
         })
         .collect();
 
-    let prefix = prefix.as_ref();
     writer.add_directory(prefix)?;
 
     let cargo_toml_path = prefix.join(manifest_path.file_name().unwrap());
 
+    let readme_name = readme
+        .as_ref()
+        .map(|readme| {
+            readme
+                .file_name()
+                .and_then(OsStr::to_str)
+                .with_context(|| format!("Missing readme filename for {}", manifest_path.display()))
+        })
+        .transpose()?;
+
     if root_crate {
-        let rewritten_cargo_toml = rewrite_cargo_toml(manifest_path, known_path_deps)?;
+        let mut document = parse_toml_file(manifest_path, "Cargo.toml")?;
+        rewrite_cargo_toml_readme(&mut document, manifest_path, readme_name)?;
+        rewrite_cargo_toml(&mut document, manifest_path, known_path_deps)?;
         writer.add_bytes(
             cargo_toml_path,
             Some(manifest_path),
-            rewritten_cargo_toml.as_bytes(),
+            document.to_string().as_bytes(),
         )?;
     } else if !skip_cargo_toml {
-        writer.add_file(cargo_toml_path, manifest_path)?;
+        let mut document = parse_toml_file(manifest_path, "Cargo.toml")?;
+        rewrite_cargo_toml_readme(&mut document, manifest_path, readme_name)?;
+        writer.add_bytes(
+            cargo_toml_path,
+            Some(manifest_path),
+            document.to_string().as_bytes(),
+        )?;
     }
 
     for (target, source) in target_source {
@@ -399,54 +471,20 @@ fn add_cargo_package_files_to_sdist(
 
     // Add local path dependencies
     for (name, path_dep) in known_path_deps.iter() {
-        debug!(
-            "Adding path dependency: {} at {}",
-            name,
-            path_dep.manifest_path.display()
-        );
-        let path_dep_manifest_dir = path_dep.manifest_path.parent().unwrap();
-        let relative_path_dep_manifest_dir =
-            path_dep_manifest_dir.strip_prefix(&sdist_root).unwrap();
-        // we may need to rewrite workspace Cargo.toml later so don't add it to sdist yet
-        let skip_cargo_toml = workspace_manifest_path == path_dep.manifest_path;
-        add_crate_to_source_distribution(
+        add_path_dep(
             writer,
-            &path_dep.manifest_path,
-            &root_dir.join(relative_path_dep_manifest_dir),
+            root_dir,
+            workspace_root,
+            &workspace_manifest_path,
             &known_path_deps,
-            false,
-            skip_cargo_toml,
+            &sdist_root,
+            name,
+            path_dep,
         )
-        .with_context(|| {
-            format!(
-                "Failed to add local dependency {} at {} to the source distribution",
-                name,
-                path_dep.manifest_path.display()
-            )
-        })?;
-        // Handle possible relative readme field in Cargo.toml
-        if let Some(readme) = path_dep.readme.as_ref() {
-            let readme = path_dep_manifest_dir.join(readme);
-            let abs_readme = readme
-                .normalize()
-                .with_context(|| format!("failed to normalize readme path `{}`", readme.display()))?
-                .into_path_buf();
-            let relative_readme = abs_readme.strip_prefix(&sdist_root).unwrap();
-            writer.add_file(root_dir.join(relative_readme), &abs_readme)?;
-        }
-        // Handle different workspace manifest
-        if &path_dep.workspace_root != workspace_root {
-            let path_dep_workspace_manifest = path_dep.workspace_root.join("Cargo.toml");
-            let relative_path_dep_workspace_manifest = path_dep_workspace_manifest
-                .strip_prefix(&sdist_root)
-                .unwrap();
-            writer.add_file(
-                root_dir.join(relative_path_dep_workspace_manifest),
-                &path_dep_workspace_manifest,
-            )?;
-        }
+        .with_context(|| format!("Failed to add path dependency {}", name))?;
     }
 
+    debug!("Adding the main crate {}", manifest_path.display());
     // Add the main crate
     let abs_manifest_path = manifest_path
         .normalize()
@@ -468,6 +506,7 @@ fn add_cargo_package_files_to_sdist(
         writer,
         manifest_path,
         root_dir.join(relative_main_crate_manifest_dir),
+        None,
         &known_path_deps,
         true,
         false,
@@ -479,8 +518,14 @@ fn add_cargo_package_files_to_sdist(
             .normalize()
             .with_context(|| format!("failed to normalize readme path `{}`", readme.display()))?
             .into_path_buf();
-        let relative_readme = abs_readme.strip_prefix(&sdist_root).unwrap();
-        writer.add_file(root_dir.join(relative_readme), &abs_readme)?;
+        // Add readme next to Cargo.toml so we don't get collisions between crates using readmes
+        // higher up the file tree.
+        writer.add_file(
+            root_dir
+                .join(relative_main_crate_manifest_dir)
+                .join(readme.file_name().unwrap()),
+            &abs_readme,
+        )?;
     }
 
     // Add Cargo.lock file and workspace Cargo.toml
@@ -524,11 +569,17 @@ fn add_cargo_package_files_to_sdist(
                     readme: None,
                 },
             );
-            let workspace_cargo_toml = rewrite_cargo_toml(&workspace_manifest_path, &deps_to_keep)?;
+            let mut document =
+                parse_toml_file(workspace_manifest_path.as_std_path(), "Cargo.toml")?;
+            rewrite_cargo_toml(
+                &mut document,
+                workspace_manifest_path.as_std_path(),
+                &deps_to_keep,
+            )?;
             writer.add_bytes(
                 root_dir.join(relative_workspace_cargo_toml),
                 Some(workspace_manifest_path.as_std_path()),
-                workspace_cargo_toml.as_bytes(),
+                document.to_string().as_bytes(),
             )?;
         }
     } else if cargo_lock_required {
@@ -597,6 +648,72 @@ fn add_cargo_package_files_to_sdist(
     Ok(())
 }
 
+#[allow(clippy::too_many_arguments)] // TODO(konsti)
+fn add_path_dep(
+    writer: &mut SDistWriter,
+    root_dir: &Path,
+    workspace_root: &Utf8Path,
+    workspace_manifest_path: &Utf8Path,
+    known_path_deps: &HashMap<String, PathDependency>,
+    sdist_root: &Path,
+    name: &str,
+    path_dep: &PathDependency,
+) -> Result<()> {
+    debug!(
+        "Adding path dependency: {} at {}",
+        name,
+        path_dep.manifest_path.display()
+    );
+    let path_dep_manifest_dir = path_dep.manifest_path.parent().unwrap();
+    let relative_path_dep_manifest_dir = path_dep_manifest_dir.strip_prefix(sdist_root).unwrap();
+    // we may need to rewrite workspace Cargo.toml later so don't add it to sdist yet
+    let skip_cargo_toml = workspace_manifest_path == path_dep.manifest_path;
+    add_crate_to_source_distribution(
+        writer,
+        &path_dep.manifest_path,
+        root_dir.join(relative_path_dep_manifest_dir),
+        path_dep.readme.as_deref(),
+        known_path_deps,
+        false,
+        skip_cargo_toml,
+    )
+    .with_context(|| {
+        format!(
+            "Failed to add local dependency {} at {} to the source distribution",
+            name,
+            path_dep.manifest_path.display()
+        )
+    })?;
+    // Include readme
+    if let Some(readme) = path_dep.readme.as_ref() {
+        let readme = path_dep_manifest_dir.join(readme);
+        let abs_readme = readme
+            .normalize()
+            .with_context(|| format!("failed to normalize readme path `{}`", readme.display()))?
+            .into_path_buf();
+        // Add readme next to Cargo.toml so we don't get collisions between crates using readmes
+        // higher up the file tree. See also [`rewrite_cargo_toml_readme`].
+        writer.add_file(
+            root_dir
+                .join(relative_path_dep_manifest_dir)
+                .join(readme.file_name().unwrap()),
+            &abs_readme,
+        )?;
+    }
+    // Handle different workspace manifest
+    if path_dep.workspace_root != workspace_root {
+        let path_dep_workspace_manifest = path_dep.workspace_root.join("Cargo.toml");
+        let relative_path_dep_workspace_manifest = path_dep_workspace_manifest
+            .strip_prefix(sdist_root)
+            .unwrap();
+        writer.add_file(
+            root_dir.join(relative_path_dep_workspace_manifest),
+            &path_dep_workspace_manifest,
+        )?;
+    }
+    Ok(())
+}
+
 /// Creates a source distribution, packing the root crate and all local dependencies
 ///
 /// The source distribution format is specified in
