@@ -1,4 +1,6 @@
 use crate::cross_compile::is_cross_compiling;
+use crate::python_interpreter::InterpreterKind;
+use crate::python_interpreter::InterpreterKind::{CPython, GraalPy, PyPy};
 use crate::PlatformTag;
 use anyhow::{anyhow, bail, format_err, Result};
 use platform_info::*;
@@ -33,6 +35,7 @@ pub enum Os {
     Emscripten,
     Wasi,
     Aix,
+    Hurd,
 }
 
 impl fmt::Display for Os {
@@ -52,6 +55,7 @@ impl fmt::Display for Os {
             Os::Emscripten => write!(f, "Emscripten"),
             Os::Wasi => write!(f, "Wasi"),
             Os::Aix => write!(f, "AIX"),
+            Os::Hurd => write!(f, "Hurd"),
         }
     }
 }
@@ -191,6 +195,7 @@ fn get_supported_architectures(os: &Os) -> Vec<Arch> {
         Os::Solaris => vec![Arch::X86_64, Arch::Sparc64],
         Os::Emscripten | Os::Wasi => vec![Arch::Wasm32],
         Os::Aix => vec![Arch::Powerpc64],
+        Os::Hurd => vec![Arch::X86, Arch::X86_64],
     }
 }
 
@@ -253,6 +258,7 @@ impl Target {
             OperatingSystem::Emscripten => Os::Emscripten,
             OperatingSystem::Wasi | OperatingSystem::WasiP1 | OperatingSystem::WasiP2 => Os::Wasi,
             OperatingSystem::Aix => Os::Aix,
+            OperatingSystem::Hurd => Os::Hurd,
             unsupported => bail!("The operating system {:?} is not supported", unsupported),
         };
 
@@ -368,6 +374,44 @@ impl Target {
         }
     }
 
+    /// Returns the extension architecture name python uses in `ext_suffix` for this architecture.
+    pub fn get_python_ext_arch(&self, python_impl: InterpreterKind) -> &str {
+        if matches!(self.target_arch(), Arch::Armv6L | Arch::Armv7L) {
+            "arm"
+        } else if matches!(self.target_arch(), Arch::Powerpc64Le)
+            && python_impl == InterpreterKind::PyPy
+        {
+            "ppc_64"
+        } else if matches!(self.target_arch(), Arch::X86) && python_impl == InterpreterKind::PyPy {
+            "x86"
+        } else if matches!(self.target_arch(), Arch::Powerpc) {
+            "powerpc"
+        } else {
+            self.get_python_arch()
+        }
+    }
+
+    /// Returns the environment python uses in `ext_suffix` for this architecture.
+    pub fn get_python_target_env(
+        &self,
+        python_impl: InterpreterKind,
+        python_version: (usize, usize),
+    ) -> String {
+        match python_impl {
+            CPython => {
+                // For musl handling see https://github.com/pypa/auditwheel/issues/349
+                if matches!(self.target_arch(), Arch::Mips64 | Arch::Mips64el) && self.is_linux() {
+                    "gnuabi64".to_string()
+                } else if python_version >= (3, 11) {
+                    self.target_env().to_string()
+                } else {
+                    self.target_env().to_string().replace("musl", "gnu")
+                }
+            }
+            PyPy | GraalPy => "gnu".to_string(),
+        }
+    }
+
     /// Returns the name python uses in `sys.platform` for this os
     pub fn get_python_os(&self) -> &str {
         match self.os {
@@ -386,6 +430,7 @@ impl Target {
             // This isn't real, there's no sys.platform here
             Os::Wasi => "wasi",
             Os::Aix => "aix",
+            Os::Hurd => "gnu",
         }
     }
 
@@ -470,7 +515,8 @@ impl Target {
             | Os::Haiku
             | Os::Emscripten
             | Os::Wasi
-            | Os::Aix => true,
+            | Os::Aix
+            | Os::Hurd => true,
         }
     }
 
@@ -546,6 +592,12 @@ impl Target {
         self.os == Os::Wasi
     }
 
+    /// Returns true if we're building a binary for GNU/Hurd
+    #[inline]
+    pub fn is_hurd(&self) -> bool {
+        self.os == Os::Hurd
+    }
+
     /// Returns true if the current platform's target env is Musl
     #[inline]
     pub fn is_musl_libc(&self) -> bool {
